#!/usr/bin/perl -w

# must be run from within project directory. E.g. ~/projects/cplan

# -assign flag forces work units to be assign to cloud worker when create_work is called.

# E.g.
# Create low priority workunits
# ./create_test_work.pl -appname worker -numwu 10 -batch 0 -label "lowpri" -db clouds -password "" -wu "templates/worker_wu" -result "templates/worker_result"

# Create high priority workunits that are immediately scheduled on cloud
# workers by default (no need to call assign_batch_to_cloud.php).
# Removing the -assign flag would allow one to then test dynamic
# assignment of wus to cloud workers by calling assign_batch_cloud.php
# ./create_test_work.pl -appname worker -numwu 10 -batch 1 -label "highpri" -db clouds -password "" -wu "templates/worker_wu" -result "templates/worker_result" -assign

use DBI;
use strict;

sub main();
sub create_wu($$$$$$$$);
sub extract_parameters();
use Getopt::Long;

#############################################################################
my ($CLOUD_USER_ID) = 1;
#############################################################################

main;

sub main(){



    my ($appname, $numwu, $batch, $label, $assign, $user, $db, $password,
    $wu_template, $result_template) = extract_parameters();
    my ($dsn, $dbh, $sth, $i);

    $dsn = "DBI:mysql:$db:localhost";

# Connecting is exppensive.
# Auto commit so that changes are automatically saved when executed
    $dbh = DBI->connect ($dsn, $user, $password,
                         { PrintError => 1, RaiseError => 1, AutoCommit => 0 });


    print "# of workunits to create: ", $numwu, "\n\n";

    for ($i=0; $i<$numwu; $i++){

        $sth = $dbh->prepare ("SELECT MAX(id) FROM workunit");

        create_wu($sth, $i, $appname, $batch, $label, $assign,
        $wu_template, $result_template);

    }

# reload the feeder
    if ($assign){

        system ('touch reread_db');
    }

    $dbh->disconnect;

}

sub create_wu ($$$$$$$$){
    my ($sth, $i, $appname, $batch, $label, $assign, $wu_template, $result_template) = @_;

    my ($new_id, $id, $cmd);

    $sth->execute();

    while (my @row = $sth->fetchrow_array){
        $id = $row[0];
    }

    $id=1000 if not defined($id);

    $new_id = $id + 1 + $i;

    my ($wu_name) = "testwu" . (defined($label)?
                                "_${label}_": "")
        . ($assign? "asgn_": "") . $new_id;


    my($batch_flag) =  ($batch? "-batch $batch": "");
    my ($asgn_flag) = ($assign? "--assign_cloud_one $CLOUD_USER_ID": "");


# put input file in the hierarchical directory
    my($download_path) = qx(./bin/dir_hier_path in);
    chomp($download_path);

    if (!(-e $download_path)){
        print "download path = $download_path\n";
    	if (!(-e "in")){
	    system("echo 'Dummy input file' > in")
	}
        $cmd = "cp in $download_path";
        print $cmd, "\n";
        system($cmd) == 0 or die "cp failed";
    }

    print "new wu name: $wu_name, wu id: $new_id, batch: $batch, label: " . (defined($label)? $label : "") . "\n";

    $cmd = "./bin/create_work -appname $appname -wu_name $wu_name -wu_template $wu_template -result_template $result_template --min_quorum 1 $batch_flag $asgn_flag in";

    print $cmd, "\n\n";

    system("$cmd") == 0 or die "$cmd failed";

}


sub extract_parameters()
{
	my %params = (
		numwu => \(my $numwu = undef),
        batch => \(my $batch = undef),
        label => \(my $label = undef),
        user => \(my $user = undef),
        db => \(my $db = undef),
        password => \(my $password = undef),
        assign => \(my $assign = 0),
        appname => \(my $appname = undef),
        wu_template => \(my $wu_template = undef),
        result_template => \(my $result_template = undef),
	);

	# Call Library function which will extract and remove all switches and
	# their corresponding values.
	# These parameters will be removed from @ARGV

	my $errorval = &GetOptions(\%params, "numwu=s", "db=s",
                               "batch=s", "label=s", "user=s", "password=s",
                               "assign!", "appname=s", "wu_template=s", "result_template=s");
                        # the exclaimation point allows for the negation
                        # of the switch (i.e. -nobackup/-nobody is a switch)

	# Error value should have returned 1 if nothing went wrong
	# Otherwise, an unlisted parameter was specified.
	if($errorval !=1)
	{
		# Expecting GetOptions to state error.

		print "Exiting Program...\n";
		exit 0;
	}

	if(!(defined ($appname))){
	   $appname = "uppercase";
	}

	if(!(defined ($user))){
	   $user = "boincadm";
	}

	if(!(defined ($db))){
	   $db = "boinc_stp_db";
	}

	if(!(defined ($numwu))){
	    $numwu = "1";
	}

	if(!(defined ($batch))){
	    $batch = "0";
	}

	if(!(defined ($label))){
	    $label = undef;
	}

	if(!(defined ($password))){
	    $password = "edgi";
	}

	if(!(defined ($wu_template))){
	    $wu_template = "templates/uc_wu";
	}

	if(!(defined ($result_template))){
	    $result_template = "templates/uc_result";
	}

	return ($appname, $numwu, $batch, $label, $assign, $user, $db,
	$password, $wu_template, $result_template);

}
