#! /usr/bin/python

"""Unit tests for order.py

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
sys.path.append(os.path.join(sys.path[0],"..",".."))
os.chdir(os.path.join(sys.path[0],".."))

from tools import db_func
from order import order
import unittest
import random

DEFAULT_BATCH_ID="batch_0"
DEFAULT_USER_ID=0


class BadArgument(unittest.TestCase):
    """Test for bad arguments

    
    """
    
    def testNonStrBatchId(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        batch_id = random.randint(0,1000)
        self.assertEqual(order(conn,batch_id,DEFAULT_USER_ID,1), None)
        db_func.disconnect(conn)
    
    def testWrongDBConn(self):
        conn=db_func.connect(db_func.read_db_config("../scheduler/db.cfg"))
        self.assertEqual(order(conn,DEFAULT_BATCH_ID,DEFAULT_USER_ID,1), None)
        db_func.disconnect(conn)

    def testNoDBConn(self):
        self.assertEqual(order(None,DEFAULT_BATCH_ID,DEFAULT_USER_ID,1), None)

    def testNegativeCredit(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        credit = -random.randint(0,1000)
        self.assertEqual(order(conn,DEFAULT_BATCH_ID,DEFAULT_USER_ID,credit), None)
        db_func.disconnect(conn)
    
    def testZeroCredit(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        credit = 0
        self.assertEqual(order(conn,DEFAULT_BATCH_ID,DEFAULT_USER_ID,credit), None)
        db_func.disconnect(conn)
                
    def testNonIntCredit(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        credit = str(random.randint(0,1000))
        self.assertEqual(order(conn,DEFAULT_BATCH_ID,DEFAULT_USER_ID,credit), None)
        db_func.disconnect(conn)

class BadReturn(unittest.TestCase):
    """Test for bad returned values
    
    """
    
    def testReturnAreBool(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))    
        res = order(conn,DEFAULT_BATCH_ID,DEFAULT_USER_ID,1)
        self.assert_(isinstance(res, bool))
        db_func.disconnect(conn)


class ConsistencyCheck(unittest.TestCase):
    """Test consistency
    
    """
    def testOrderShouldEnd(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))        
        for i in range(0,1000):
            if not order(conn,DEFAULT_BATCH_ID,DEFAULT_USER_ID,1):
                break
        self.assert_(i<999)
        db_func.disconnect(conn)   
    
            
if __name__ == "__main__":
    unittest.main()   
