#! /usr/bin/python

"""//Tools to grab batch completion from an XWHEP DesktopGrid//

- > It inserts the information in the QoSInfo database table "BatchCompletionHistory".

- File: grabber_XWHEP_bots.py 
- Usage: ./grabber_XWHEP_bots.py URL

- Command line argument:
 - URL : The URL of the XWHEP batch completion history wrapper page (i.e. http://<XWHEP server address/spequlos/XWHEP-wrapper.php)

- Notes: 
 - A database configuration file named "db.cfg" must be present in the directory.

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import conf_func
from tools import db_func
from tools import log_func
from tools import web_func
log=log_func.get_spequlos_log(__file__,html=False)

import urllib 
import time

def grab(conn,url):
    """Grab information from a XWHEP batch completion Web page and insert it into the database. 
         
    - Arguments:
     - conn -- The Info's DB connection, as a MySQL.connections.Connection object
     - url -- The of the XWHEP page, as a string
  
    """    
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None     
    if not isinstance(url,str):
        log.error("The url argument must be a string !")
        return None
   
    c=conn.cursor()
    
    log.info("Grabbing information on url %s",url)
    res = web_func.call(url)

    if len(res)==0:
        log.error("No result found at url %s",url)
        return None

    n_compl={}
    n_uncompl={}
    n_submitted={}

    for r in res:
        data = r.split()        
        try:
            if len(data)<3:
                raise ValueError("Not enough columns in the page")
            batch_id=data[0]
            status=data[1]
            n=int(data[2])
        except ValueError as e:
            log.error("Line contains some invalid data structure : %s" % r)
            log.error(e)
            log.error("Ignoring it")
            break
        

        log.info("Processing line : %s %s %d" % (batch_id,status,n))

        if not batch_id in n_compl:
            n_compl[batch_id]=0
            n_uncompl[batch_id]=0
            n_submitted[batch_id]=0

        if status=="COMPLETED" or status=="ERROR":
            n_compl[batch_id]+=n

        if status=="RUNNING" or status=="PENDING" or status=="WAITING" or status=="ABORDED" or status=="LOST":
            n_uncompl[batch_id]+=n
    
        if status=="RUNNING" or status=="COMPLETED" or status=="ERROR":
            n_submitted[batch_id]+=n


    ts = time.time()
    for batch_id in n_compl.keys():
        log.info("Checking if completion history changed since last database insertion")
        try:
            c.execute("SELECT n_jobs_completed, n_jobs_uncompleted FROM BatchCompletionHistory WHERE batch_id='%s' AND ts=\
                (SELECT MAX(ts) FROM BatchCompletionHistory WHERE batch_id='%s')" % (batch_id,batch_id))
        except db_func.DBError as e:
            log.error("Error during the SQL command execution !")
            log.error(e)
            return None    

        res = c.fetchall()
        
        if len(res)>1:
            log.error("Several results with same timestamp and batch_id %s in the database ! '(ts,batch_id)' should be BatchCompletionHistory PRIMARY KEY ! "% batch_id)
            return None
    
        if len(res)==0:
            log.info("No entry with such batch identifier in the database : %s" % batch_id )
        else:
            try:
                last_n_compl=int(res[0][0])
                last_n_uncompl=int(res[0][1])
            except ValueError as e:
                log.error("The 'n_jobs_(un)completed' in BatchCompletionHistory table must be a positive or zero integer !")
                log.error(e)
                return None        

            if n_compl[batch_id]!=last_n_compl or n_uncompl[batch_id]!=last_n_uncompl:
                log.info("BatchCompletionHistory has changed for batch %s" % batch_id)
            else:
                log.info("BatchCompletionHistory has not changed for batch %s, do not need to insert data" % batch_id)
                continue
                    
        log.info("Inserting information in database for batch %s" % batch_id)
        try:
            c.execute("INSERT INTO BatchCompletionHistory (ts,batch_id,n_jobs_completed,n_jobs_uncompleted,n_jobs_submitted,plugin_addr) VALUES (%d,'%s',%d,%d,%d,'%s')" \
                    % (ts,batch_id,n_compl[batch_id],n_uncompl[batch_id],n_submitted[batch_id],url))
        except db_func.DBError as e:
            log.error("Error during the SQL command execution !")
            log.error(e)
            return None  

if __name__=="__main__":

    if len(sys.argv)!=2:
        print """Usage: 
grabber_XWHEP.py <url>"""
        sys.exit(2)
   
    url=sys.argv[1]
   
    conn=db_func.connect(db_func.read_db_config("db.cfg"))
    grab(conn,url)
    db_func.disconnect(conn)


