#!/bin/sh

cd $(dirname $0)
. ./install.conf

tools_install () {
    echo "Installing SpeQulos tools in $INSTALL_DIR/tools ..."
    mkdir -p $INSTALL_DIR/tools
    cp tools/*.py $INSTALL_DIR/tools

    echo "SpeQuloS tools installation completed\n"
}

info_install () {
    echo "Installing SpeQulos Information module in $INSTALL_DIR/info ..."
    mkdir -p $INSTALL_DIR/info
    cp info/*.py info/*.cfg $INSTALL_DIR/info
    echo "host=localhost
user=$INFO_DB_USER
db=$INFO_DB_NAME
passwd=$INFO_DB_PASS" > $INSTALL_DIR/info/db.cfg

    echo "Creating Info module database $INFO_DB_NAME ..."
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "DROP DATABASE IF EXISTS $INFO_DB_NAME;"
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "CREATE database $INFO_DB_NAME;"
    #mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "CREATE USER $INFO_DB_USER;"
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "GRANT ALL PRIVILEGES ON $INFO_DB_NAME.* TO $INFO_DB_USER IDENTIFIED BY '$INFO_DB_PASS';"

    echo "Setting up database $INFO_DB_NAME ..."
    mysql -u $INFO_DB_USER --password="$INFO_DB_PASS" $INFO_DB_NAME < info/info_db.sql

    
    echo "SpeQuloS Information module installation completed\n"
}

oracle_install () {
    echo "Installing SpeQulos Oracle module in $INSTALL_DIR/oracle ..."
    mkdir -p $INSTALL_DIR/oracle
    cp oracle/*.py oracle/*.cfg $INSTALL_DIR/oracle
    echo "host=localhost
user=$INFO_DB_USER
db=$INFO_DB_NAME
passwd=$INFO_DB_PASS" > $INSTALL_DIR/oracle/info_db.cfg

    echo "Setting up Web services in $HTTP_DIR/oracle ..."
    mkdir -p $HTTP_DIR/oracle
    ln -sf $INSTALL_DIR/oracle/calculate_cloud_benefit.py $HTTP_DIR/oracle/
    ln -sf $INSTALL_DIR/oracle/get_elapsed_time.py $HTTP_DIR/oracle/
    ln -sf $INSTALL_DIR/oracle/get_n_jobs.py $HTTP_DIR/oracle/
    ln -sf $INSTALL_DIR/oracle/get_percent_completed.py $HTTP_DIR/oracle/

    echo "SpeQuloS Oracle module installation completed\n"       
}

scheduler_install () {
    echo "Installing SpeQulos Scheduler module in $INSTALL_DIR/scheduler ..."
    mkdir -p $INSTALL_DIR/scheduler
    cp scheduler/*.py scheduler/*.cfg $INSTALL_DIR/scheduler
    mkdir -p $INSTALL_DIR/scheduler/libcloud_cloudworker
    cp scheduler/libcloud_cloudworker/*.py $INSTALL_DIR/scheduler/libcloud_cloudworker
    cp scheduler/libcloud_cloudworker/*.cfg scheduler/libcloud_cloudworker/id_rsa* $INSTALL_DIR/scheduler/
    echo "host=localhost
user=$SCHEDULER_DB_USER
db=$SCHEDULER_DB_NAME
passwd=$SCHEDULER_DB_PASS" > $INSTALL_DIR/scheduler/db.cfg

    echo "Setting up Web services in $HTTP_DIR/scheduler ..."
    mkdir -p $HTTP_DIR/scheduler
    ln -sf $INSTALL_DIR/scheduler/start_qos.py $HTTP_DIR/scheduler/
    ln -sf $INSTALL_DIR/scheduler/admin_cloud.py $HTTP_DIR/scheduler/
    ln -sf $INSTALL_DIR/scheduler/admin_dg.py $HTTP_DIR/scheduler/


    echo "Creating Scheduler module database $SCHEDULER_DB_NAME ..."
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "DROP DATABASE IF EXISTS $SCHEDULER_DB_NAME;"
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "CREATE database $SCHEDULER_DB_NAME;"
    #mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "CREATE USER $SCHEDULER_DB_USER;"
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "GRANT ALL PRIVILEGES ON $SCHEDULER_DB_NAME.* TO $SCHEDULER_DB_USER IDENTIFIED BY '$SCHEDULER_DB_PASS';"

    echo "Setting up database $SCHEDULER_DB_NAME ..."
    mysql -u $SCHEDULER_DB_USER --password="$SCHEDULER_DB_PASS" $SCHEDULER_DB_NAME < scheduler/scheduler_db.sql

    #echo "Setting up periodic execution with cron ..."
    #echo "0-59/4 * * * * root $INSTALL_DIR/scheduler/monitor_batches.py >> /tmp/monitor_batches.log
    #2-59/4 * * * * root $INSTALL_DIR/scheduler/monitor_cloud_workers.py >> /tmp/monitor_cloud_workers.log" > /etc/cron.d/spequlos_scheduler

    echo "SpeQuloS Scheduler module installation completed\n"     
}

creditsystem_install () {
    echo "Installing SpeQulos CreditSystem module in $INSTALL_DIR/creditsystem ..."
    mkdir -p $INSTALL_DIR/creditsystem
    cp creditsystem/*.py creditsystem/*.cfg $INSTALL_DIR/creditsystem

    echo "Setting up Web services in $HTTP_DIR/creditsystem ..."
    mkdir -p $HTTP_DIR/creditsystem
    ln -sf $INSTALL_DIR/creditsystem/bill.py $HTTP_DIR/creditsystem
    ln -sf $INSTALL_DIR/creditsystem/deposit.py $HTTP_DIR/creditsystem
    ln -sf $INSTALL_DIR/creditsystem/get_credits.py $HTTP_DIR/creditsystem
    ln -sf $INSTALL_DIR/creditsystem/order.py $HTTP_DIR/creditsystem
    ln -sf $INSTALL_DIR/creditsystem/pay.py $HTTP_DIR/creditsystem
    ln -sf $INSTALL_DIR/creditsystem/admin_users.py $HTTP_DIR/creditsystem
    ln -sf $INSTALL_DIR/creditsystem/admin_institutions.py $HTTP_DIR/creditsystem

    echo "Creating CreditSystem module database $CREDITSYSTEM_DB_NAME ..."
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "DROP DATABASE IF EXISTS $CREDITSYSTEM_DB_NAME;"
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "CREATE database $CREDITSYSTEM_DB_NAME;"
    #mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "CREATE USER $CREDITSYSTEM_DB_USER;"
    mysql -u $MYSQL_ADMIN --password="$MYSQL_PASS" -e "GRANT ALL PRIVILEGES ON $CREDITSYSTEM_DB_NAME.* TO $CREDITSYSTEM_DB_USER IDENTIFIED BY '$CREDITSYSTEM_DB_PASS';"

    echo "Setting up database $CREDITSYSTEM_DB_NAME ..."
    mysql -u $CREDITSYSTEM_DB_USER --password="$CREDITSYSTEM_DB_PASS" $CREDITSYSTEM_DB_NAME < creditsystem/creditsystem_db.sql

    #Database preconfiguration (should be removed later
    mysql -u $CREDITSYSTEM_DB_USER --password="$CREDITSYSTEM_DB_PASS" -e "INSERT INTO QoSUser VALUES (0,0,0);" $CREDITSYSTEM_DB_NAME 
    mysql -u $CREDITSYSTEM_DB_USER --password="$CREDITSYSTEM_DB_PASS" -e "INSERT INTO Institution VALUES (0,0,1000,1000,0);" $CREDITSYSTEM_DB_NAME 
    mysql -u $CREDITSYSTEM_DB_USER --password="$CREDITSYSTEM_DB_PASS" -e "INSERT INTO Cloud VALUES (0,0);" $CREDITSYSTEM_DB_NAME 

    echo "SpeQuloS CreditSystem module installation completed\n"        
}

apache_config () {
    echo "Configuring Apache for CGI..."
    if [ ! -d /etc/apache2/sites-enabled/ ]
    then
        echo "Could not find Apache configuration directory at /etc/apache2/sites-enabled/"
        echo "CGI must be enabled manually for .py files in $HTTP_DIR"
    else
        echo "<Directory $HTTP_DIR>
Options +ExecCGI
AddHandler cgi-script .py
</Directory>" > /etc/apache2/sites-enabled/010-spequlos
        apache2ctl restart
    fi
    echo "Apache configured"
}

case $1 in
    all)
        tools_install
        info_install
        oracle_install
        scheduler_install
        creditsystem_install
        apache_config
        ;;
    info)       
        tools_install
        info_install
        ;;
    
    oracle)
        tools_install
        oracle_install
        apache_config
        ;;

    scheduler)
        tools_install
        scheduler_install
        apache_config
        ;;

    creditsystem)
        tools_install
        creditsystem_install
        apache_config
        ;;
    *)
        echo "Usage: ./install.sh all|info|oracle|scheduler|creditsystem (as root)"
        exit 1
        ;;
esac

exit 0


