#! /usr/bin/python

"""//Web service to know the number of jobs in a batch//
 
- File: get_n_jobs.py

- Arguments - Using the HTTP GET method:
 - batch_id --- The batch identifier, as a string
 - > If the batch_id argument is missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">N</div>`` with N the number of jobs.
 
- Notes: 
 - A database configuration file named "info_db.cfg" must be present in the directory to connect to the SpeQuloS Info module database.

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import cgi_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)



def get_n_jobs(conn,batch_id):
    """Function implementing the get_n_jobs operation

    Arguments:
    conn -- The Info's DB connection, as a MySQL.connections.Connection object
    batch_id -- The batch identifier, as a string
     
    The function returns the number of jobs in the batch, as an integer 

    """
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None    
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string !")
        return None
                       
    c=conn.cursor()
    log.info("Retrieving the number of jobs in the batch")
    
    try:
        c.execute("SELECT n_jobs_completed,n_jobs_uncompleted FROM BatchCompletionHistory WHERE batch_id='%s' AND ts=\
          (SELECT MAX(ts) FROM BatchCompletionHistory WHERE batch_id='%s')" % (batch_id,batch_id))
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None            
   
    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such batch identifier in the database : %s" % batch_id )
        return None
    elif len(res)>1:
        log.error("Several results with same timestamp and batch_id %s in the database ! '(ts,batch_id)' should be BatchCompletionHistory PRIMARY KEY ! "% batch_id)
        return None
    
    try:
        n_jobs=int(res[0][0])+int(res[0][1])
        if n_jobs<0:
            raise ValueError("Negative number of jobs !")
    except (ValueError,TypeError) as e:
        log.error("The 'n_jobs_completed' and 'n_jobs_uncompleted' in BatchCompletionHistory table must be a positive or zero integer !")
        log.error(e)
        return None
    
    log.info("Number of jobs: %d" % (n_jobs))
    return n_jobs     

if __name__=="__main__":
    cgi_func.print_header()

    if cgi_func.get_form_value('batch_id') is None:
        cgi_func.print_form(['batch_id'])

    else:
        batch_id = cgi_func.get_form_value('batch_id')
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        cgi_func.print_result( get_n_jobs(conn,batch_id))
        db_func.disconnect(conn)
        
    cgi_func.print_footer()

