#! /usr/bin/python

"""Unit tests for calculate_cloud_benefit.py

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
sys.path.append(os.path.join(sys.path[0],"..",".."))
os.chdir(os.path.join(sys.path[0],".."))

from tools import db_func
from calculate_cloud_benefit import calculate_cloud_benefit
import unittest
import random
import datetime
import time 

DEFAULT_BATCH_ID = "batch_0"

class BadArgument(unittest.TestCase):
    """Test for bad argumentssys.path.append("..")

    
    """
    
    def testUnknownBatchId(self):
        batch_id = str(random.randint(0,1000))
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(calculate_cloud_benefit(conn,batch_id), None)
        db_func.disconnect(conn)
        
    def testNonStrBatchId(self):
        batch_id = random.randint(0,1000)
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(calculate_cloud_benefit(conn,batch_id), None)
        db_func.disconnect(conn)
        

    def testNegativeCurrentTime(self):
        current_time = -random.randint(0,1000)
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,current_time), None)        
        db_func.disconnect(conn)
        
    def testNonIntCurrentTime(self):
        current_time = str(random.randint(0,1000))
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,current_time), None)        
        db_func.disconnect(conn)
        
    def testNonBoolUseLastDBEntry(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry="Test"), None)        
        db_func.disconnect(conn)


class BadReturn(unittest.TestCase):
    """Test for bad returned values
    
    """
    
    def testReturnLen(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(len(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID)), 4)        
        self.assertEqual(len(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,1290176137)), 4)        
        self.assertEqual(len(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry=True)), 4)        
        db_func.disconnect(conn)

    def testTailLongerThanNonTail(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID)
        self.assert_(res[0]<res[1])
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,1290176137)        
        self.assert_(res[0]<res[1])
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry=True)            
        self.assert_(res[0]<res[1])
        db_func.disconnect(conn)
        
    def testTailLongerThanCloud(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID)
        self.assert_(res[2]<res[1])        
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,1290176137)
        self.assert_(res[2]<res[1])        
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry=True)
        self.assert_(res[2]<res[1])        
        db_func.disconnect(conn)
        
    def testResultsArePositiveInt(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID)
        for r in res:
            self.assert_(isinstance(r,int))
            self.assert_(r>=0)
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,1290176137)
        for r in res:
            self.assert_(isinstance(r,int))
            self.assert_(r>=0)
        res = calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry=True)
        for r in res:
            self.assert_(isinstance(r,int))
            self.assert_(r>=0)
        db_func.disconnect(conn)
            

class ConsistencyCheck(unittest.TestCase):
    """Test consistency
    
    """

    def testSameArg(self):
        #Commented as calculate_cloud_benefit takes several seconds to compute
        #self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID), calculate_cloud_benefit(conn,DEFAULT_BATCH_ID))
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,1290176137), calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,1290176137))
        self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry=True), calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry=True))
        db_func.disconnect(conn)
        
    def testCurrentTimeIsCurrentTime(self):
        pass
        #Commented as calculate_cloud_benefit takes several seconds to compute        
        #self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,int(time.mktime(datetime.datetime.utcnow().timetuple()))), calculate_cloud_benefit(conn,DEFAULT_BATCH_ID))
        #db_func.disconnect(conn)
   
    def testUseLastDBEntryOverCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,123456,use_last_db_entry=True), calculate_cloud_benefit(conn,DEFAULT_BATCH_ID,use_last_db_entry=True))
        db_func.disconnect(conn)
   
    
    
if __name__ == "__main__":
    unittest.main()   
