#! /usr/bin/python

"""//Web service to enable QoS for a batch//
 
- > The start_qos function is the first to be called, because this function will return the batch identifier that must be used for the other QoS related operations
 
- File: start_qos.py

- Arguments - Using the HTTP GET method:
 - dg_name -- The grid technology used for the batch computation, as a string
 - > If some arguments are missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">batch_id</div>`` with batch_id the batch identifier to use in future QoS operations
 
- Notes: 
 - A database configuration file named "db.cfg" must be present in the directory.

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import cgi_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)

import scheduler_func

import uuid


def start_qos(conn,dg_name):
    """Function implementing the start_qos operation

    The function perform the necessary operations (DB insertions) to enable future managment of the batch

    Arguments:
    conn -- The scheduler's DB connection, as a MySQL.connections.Connection object
    dg_name -- The desktop grid name used for the batch computation, as a string
    
    The function returns the batch identifier, as a string
    """    
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None           
    if not isinstance(dg_name,str):
        log.error("start_qos() dg_name argument must be a string")
        return None        
            
    c=conn.cursor()
    
    log.info("Checking if this desktop grid is registered")
    try:
        c.execute("SELECT dg_name,dg_type,dg_plugin_url FROM DesktopGrid WHERE dg_name='%s'" % dg_name)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None            
    res=c.fetchall()

    if len(res)==0:
        log.error("Unable to find the desktop grid, is it registered ?")
        return None
    elif len(res)>1:
        log.error("Database error: we should have one result! 'dg_name' must be the PRIMARY KEY of DesktopGrid table")
        return None
    dg_name = res[0][0]
    dg_type = res[0][1]
    dg_plugin_url = res[0][2]
    
    #For BOINC we need a small integer for the batch_id to be conform with 'batch' field in BOINC database
    batch_id="%d" % int( uuid.uuid4().int % 0x3FF )

    log.info("Preconfiguring the DG with the batch information...")
    res=scheduler_func.preconfigure_QoS(batch_id,dg_type,dg_plugin_url) 
    if res == None:
        log.error("Error during preconfigure_QoS with %s, %s, %s" % (batch_id,dg_type,dg_plugin_url))
        batch_data=''
    elif res == True:    
        log.info("Done, no specific batch_data")
        batch_data=''
    else:
        log.info("Done, and we have data to insert in the batch_data field of the database")
        batch_data=res

    log.info("Batch_id to use : %s . Inserting QoSBatch entry in the DB..." % batch_id)
    try:
        c.execute("INSERT INTO QoSBatches(batch_id,dg_name,batch_data) VALUES ('%s','%s','%s')" % (batch_id,dg_name,batch_data))
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None            

    log.info("start_qos() completed")

    #For XWHEP, we need to submit BoT using the group UUID, contained in the last part of the batch_data 
    if dg_type=="XWHEP":
        group_uid=batch_data.split("/")[-1]
        cgi_func.print_message("The following UID must be used for XWHEP submission: %s" % group_uid)

    return batch_id


if __name__=="__main__":
    cgi_func.print_header()

    if (cgi_func.get_form_value('dg_name') is None):
        cgi_func.print_message("Please provide the name of the Desktop Grid you want to use")
        cgi_func.print_form(['dg_name'])

    else:
        dg_name = cgi_func.get_form_value('dg_name')
        
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        batch_id = start_qos(conn,dg_name)
        cgi_func.print_message("Use this identifier for your batch:")
        cgi_func.print_result(batch_id)
        db_func.disconnect(conn)
        
    cgi_func.print_footer()

