#! /usr/bin/python

"""//Module for database related operation//

- File: db_func.py

- Database used: MySQL

"""

import MySQLdb
import conf_func
import log_func
log=log_func.get_spequlos_log(__file__)

db_connection=MySQLdb._mysql.connection
DBError=MySQLdb.ProgrammingError

def read_db_config(filename):
    """//Read a database configuration file//

    - Arguments:
     - filename --- The configuration file name, as a string

    - The function returns a dictionary containing the keys: "host", "db", "user", "passwd", with the according values as read in the file.
         
    """
    params=conf_func.read_config(filename)
    if not "host" in params:
        log.error("Please inform the host option in the db configuration file ("+filename+")")
        raise Exception("No host information in database configuration file: %s" %filename)
    if not "db" in params:
        log.error("Please inform the db option in the db configuration file ("+filename+")")
        raise Exception("No database name information in database configuration file: %s" %filename)
    if not "user" in params:
        log.error("Please inform the user option in the db configuration file ("+filename+")")
        raise Exception("No user information in database configuration file: %s" %filename)
    if not "passwd" in params:
        log.error("Please inform the passwd option in the db configuration file ("+filename+")")
        raise Exception("No password information in database configuration file: %s" %filename)
    return params


def connect(params):
    """//Connect to a MySQL database//

    - Arguments:
     - params --- The database configuration parameters, as a dictionary. It must contain the right values for database access in the keys: "host", "db", "user", "passwd".

    - The function returns a MySQLdb.connections.Connection instance on the database connection
    
    """
    log.info("Connecting to database...")
    try:
        if "port" in params:
            try:
                port=int(params["port"])
            except Exception as e:
                log.error("Error when reading DB port in parameters")
                log.error(e)
                raise e
            else:
                conn = MySQLdb.connect(host=params["host"],db=params["db"],user=params["user"],passwd=params["passwd"],port=port)
        else:
            conn = MySQLdb.connect(host=params["host"],db=params["db"],user=params["user"],passwd=params["passwd"])
    except Exception as e:
        log.error("Error when connecting to database")
        log.error(e)        
        raise e
    log.info("Done !")
    return conn


def disconnect(conn):
    """//Disconnect from a connected MySQL database//

    - Arguments:
     - conn --- The database connection, as a MySQLdb.connections.Connection
        
    """    
    log.info("Disconnecting from database...")
    conn.close()
    log.info("Done !")


